/*
 * Copyright 2005-2006 Michelle Osmond Licensed under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package message.store;

import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import javax.naming.Context;
import javax.naming.InitialContext;

import message.MessageBoxKey;
import message.MessageConfig;
import message.MessageStore;
import message.ejb.PortalMessageStore;
import message.ejb.PortalMessageStoreHome;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Storage and retrieval of portlet message mappings, and the messages. They are
 * stored on a remote EJB, which this class queries. This should be accessible
 * to all portlet apps. The "java.naming.factory.url.pkgs" property used when
 * connecting to the EJB server is configurable; set it in the
 * PortletMessaging.properties.
 * <p>
 * This is a very simple implementation which serialises and stores a whole
 * MessageStore to the database - this is not particularly efficient!
 * 
 * @see message.MessageStore
 * @author Michelle Osmond
 * @version 1.3
 */
public class EJBMessageStore implements MessageStore, java.io.Serializable
{
    private static final Log log = LogFactory.getLog(EJBMessageStore.class);

    protected String sessionID;

    protected transient PortalMessageStore store;

    public EJBMessageStore()
    {
    }

    // ---- Implement MessageStore methods. ------

    public void initialise(String sessionID)
    {
        this.sessionID = sessionID;

        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, MessageConfig.getConfigProperty(Context.INITIAL_CONTEXT_FACTORY));
        env.put(Context.PROVIDER_URL, MessageConfig.getConfigProperty(Context.PROVIDER_URL));
        env.put(Context.URL_PKG_PREFIXES, MessageConfig.getConfigProperty(Context.URL_PKG_PREFIXES));
        String username = MessageConfig.getConfigProperty(Context.SECURITY_PRINCIPAL);
        String password = MessageConfig.getConfigProperty(Context.SECURITY_CREDENTIALS);
        if (username != null)
        {
            env.put(Context.SECURITY_PRINCIPAL, username);
            env.put(Context.SECURITY_CREDENTIALS, password);
        }

        try
        {
            if (log.isDebugEnabled())
            {
                log.debug("EJBMessageStore getting PortalMessageStore bean for session id " + sessionID);
            }
            Context ctx = new InitialContext(env);
            Object obj = ctx.lookup("PortalMessageStore");
            PortalMessageStoreHome home = (PortalMessageStoreHome) javax.rmi.PortableRemoteObject.narrow(obj,
                PortalMessageStoreHome.class);
            try
            {
                if (log.isDebugEnabled())
                {
                    log.debug(" - looking for existing Store");
                }
                this.store = home.findByPrimaryKey(sessionID);
            }
            catch (Exception e)
            {
                if (log.isDebugEnabled())
                {
                    log.debug(" - creating new Store");
                }
                this.store = home.create(sessionID);
            }
            if (log.isDebugEnabled())
            {
                log.debug("EJBMessageStore got PortalMessageStore bean for session id " + sessionID);
            }
        }
        catch (Exception e)
        {
            log.error("Exception initialising EJBMessageStore: " + e.getMessage(), e);
        }
    }

    /**
     * Check if the store is valid, and re-initialise if necessary & possible.
     * (e.g. if session has been serialised, we may need to retrieve the EJB
     * reference again.)
     */
    public boolean isValid()
    {
        if (store == null && sessionID != null)
        {
            initialise(sessionID);
        }
        return store != null && sessionID != null;
    }

    public void saveInputMapping(MessageBoxKey key, MessageBoxKey source)
    {
        try
        {
            store.saveInputMapping(key, source);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
        }
    }

    public void deleteInput(MessageBoxKey key)
    {
        try
        {
            store.deleteInput(key);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
        }
    }

    public void saveOutputMapping(MessageBoxKey key, MessageBoxKey target)
    {
        try
        {
            store.saveOutputMapping(key, target);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
        }
    }

    public void clearAllOutputMappings(MessageBoxKey key)
    {
        try
        {
            store.clearAllOutputMappings(key);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
        }
    }

    public void deleteOutput(MessageBoxKey key)
    {
        try
        {
            store.deleteOutput(key);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
        }
    }

    public Set findOutputTargets(MessageBoxKey key)
    {
        try
        {
            return store.findOutputTargets(key);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
            return null;
        }

    }

    public MessageBoxKey getInputSource(MessageBoxKey key)
    {
        try
        {
            return store.getInputSource(key);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    public Object getMessage(MessageBoxKey key)
    {
        try
        {
            return store.getMessage(key);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    public void sendMessage(Set targetKeys, Object message)
    {
        try
        {
            store.sendMessage(targetKeys, message);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
        }
    }

    public Set getInputKeys(String portletID)
    {
        try
        {
            return store.getInputKeys(portletID);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    public Set getOutputKeys(String portletID)
    {
        try
        {
            return store.getOutputKeys(portletID);
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    public Map getBoxes()
    {
        try
        {
            return store.getBoxes();
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    public Set getPublicBoxKeys()
    {
        try
        {
            return store.getPublicBoxKeys();
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    public Set getAllInputKeys()
    {
        try
        {
            return store.getAllInputKeys();
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    public Set getAllOutputKeys()
    {
        try
        {
            return store.getAllOutputKeys();
        }
        catch (Exception e)
        {
            log.error(e.getMessage(), e);
            return null;
        }
    }

}
