/*
 * Copyright 2005-2006 Michelle Osmond Licensed under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package message;

import java.util.Map;
import java.util.Set;

/**
 * Storage and retrieval of portlet message mappings, and the messages.
 * 
 * @author Michelle Osmond
 * @version 1.2
 * @see MessageCentreImpl
 */
public interface MessageCentre
{

    void setStore(MessageStore store);

    /**
     * Set a portlet message key => message box mapping. This will overwrite any
     * existing input mapping.
     * 
     * @param portletID
     *            ID of the portlet which is receiving the input
     * @param name
     *            (local) name of the portlet input, referred to in the
     *            portlet's code
     * @param source
     *            the message box from which the portlet's input will read.
     */
    void addInputMapping(String portletID, String name, MessageBoxKey source);

    void addInputMapping(MessageBoxKey key, MessageBoxKey source);

    /**
     * Remove a message key => message box mapping. As there can only be 1
     * mapping for each input, this will result in the default mapping: looking
     * for a local portlet message of the same name.
     */
    void clearInputMapping(MessageBoxKey key);

    /**
     * Remove all trace of the specified input message box. There will no longer
     * be even a default mapping.
     */
    void deleteInput(MessageBoxKey key);

    /**
     * Add a portlet message key => message box mapping. This will be in
     * addition to any existing output mappings; the message will be copied to
     * all mapped output targets.
     * 
     * @param portletID
     *            ID of the portlet which is sending the output
     * @param name
     *            (local) name of the portlet output, referred to in the
     *            portlet's code
     * @param target
     *            the message box to which the portlet's output will copy its
     *            message.
     */
    void addOutputMapping(String portletID, String name, MessageBoxKey target);

    void addOutputMapping(MessageBoxKey key, MessageBoxKey target);

    /**
     * Remove any existing output mappings for this key. The result will be that
     * the output still exists, but does not publish to anywhere.
     */
    void clearAllOutputMappings(MessageBoxKey key);

    /**
     * Remove all trace of the specified output message box. There will no
     * longer be even a default mapping, so it will no longer be possible to
     * publish messages to it.
     */
    void deleteOutput(MessageBoxKey key);

    /**
     * This will always return a Set which contains at least a key for a message
     * box corresponding to the provided portletID+name (it always outputs to
     * itself as well as any other registered mappings)
     * 
     * @return Set of MessageBoxKey's: the output keys registered for this
     *         portletID+name
     */
    Set findOutputTargets(String portletID, String name);

    Set findOutputTargets(MessageBoxKey key);

    /**
     * Get the key of the source message box for this portletID+name
     * combination. If no mapping is found, returns a MessageBoxKey for this
     * portletID+name.
     */
    MessageBoxKey findInputSource(String portletID, String name);

    MessageBoxKey findInputSource(MessageBoxKey key);

    /**
     * Get message for this portletID+name combination. If there is no mapping
     * to a message box defined for the specified portletID+name, it tries to
     * find a local message box of the same name. If there is no message,
     * returns null.
     */
    Object get(String portletID, String name);

    /**
     * Get message from this message box. If there is no message, returns null.
     */
    Object get(MessageBoxKey boxKey);

    /**
     * Send message to this portletID+name combination. It will be delivered to
     * a local message box with this name. It will also be delivered to all
     * message boxes for which mappings have been registered for this
     * portletID+name using addMapping. It will replace any message already in
     * the boxes.
     */
    void send(String portletID, String name, Object message);

    /**
     * Send message to these message boxes. It will replace any message already
     * in the boxes, or remove the box if 'message' is null.
     * 
     * @param targetKeys
     *            Set of MessageBoxKeys to which the message should be sent
     */
    void send(Set targetKeys, Object message);

    Set getInputKeys(String portletID);

    Set getOutputKeys(String portletID);

    Set getPublicBoxKeys();

    Map getBoxes();

    Set getAllInputKeys();

    Set getAllOutputKeys();

    boolean isValid();
}
